Index: src/arm/full-codegen-arm.cc |
diff --git a/src/arm/full-codegen-arm.cc b/src/arm/full-codegen-arm.cc |
index b9322b858dd98f0804f76d437b43d9f056170db7..ee5517cf77b9800bf6ef87d8673a54a72c2fc0df 100644 |
--- a/src/arm/full-codegen-arm.cc |
+++ b/src/arm/full-codegen-arm.cc |
@@ -1081,9 +1081,8 @@ void FullCodeGenerator::VisitForInStatement(ForInStatement* stmt) { |
ForIn loop_statement(this, stmt); |
increment_loop_depth(); |
// Get the object to enumerate over. Both SpiderMonkey and JSC |
// ignore null and undefined in contrast to the specification; see |
// ECMA-262 section 12.6.4. |
VisitForAccumulatorValue(stmt->enumerable()); |
__ LoadRoot(ip, Heap::kUndefinedValueRootIndex); |
__ cmp(r0, ip); |
@@ -1259,6 +1258,65 @@ void FullCodeGenerator::VisitForInStatement(ForInStatement* stmt) { |
} |
+void FullCodeGenerator::VisitForOfStatement(ForOfStatement* stmt) { |
+ Comment cmnt(masm_, "[ ForOfStatement"); |
+ SetStatementPosition(stmt); |
+ |
+ Iteration loop_statement(this, stmt); |
+ increment_loop_depth(); |
+ |
+ // var iterator = iterable[@@iterator]() |
+ VisitForAccumulatorValue(stmt->assign_iterator()); |
+ |
+ // As with for-in, skip the loop if the iterator is null or undefined. |
+ __ CompareRoot(r0, Heap::kUndefinedValueRootIndex); |
+ __ b(eq, loop_statement.break_label()); |
+ __ CompareRoot(r0, Heap::kNullValueRootIndex); |
+ __ b(eq, loop_statement.break_label()); |
+ |
+ // Convert the iterator to a JS object. |
+ Label convert, done_convert; |
+ __ JumpIfSmi(r0, &convert); |
+ __ CompareObjectType(r0, r1, r1, FIRST_SPEC_OBJECT_TYPE); |
+ __ b(ge, &done_convert); |
+ __ bind(&convert); |
+ __ push(r0); |
+ __ InvokeBuiltin(Builtins::TO_OBJECT, CALL_FUNCTION); |
+ __ bind(&done_convert); |
+ __ push(r0); |
+ |
+ // Loop entry. |
+ __ bind(loop_statement.continue_label()); |
+ |
+ // result = iterator.next() |
+ VisitForEffect(stmt->next_result()); |
+ |
+ // if (result.done) break; |
+ Label result_not_done; |
+ VisitForControl(stmt->result_done(), |
+ loop_statement.break_label(), |
+ &result_not_done, |
+ &result_not_done); |
+ __ bind(&result_not_done); |
+ |
+ // each = result.value |
+ VisitForEffect(stmt->assign_each()); |
+ |
+ // Generate code for the body of the loop. |
+ Visit(stmt->body()); |
+ |
+ // Check stack before looping. |
+ PrepareForBailoutForId(stmt->BackEdgeId(), NO_REGISTERS); |
+ EmitBackEdgeBookkeeping(stmt, loop_statement.continue_label()); |
+ __ jmp(loop_statement.continue_label()); |
+ |
+ // Exit and decrement the loop depth. |
+ PrepareForBailoutForId(stmt->ExitId(), NO_REGISTERS); |
+ __ bind(loop_statement.break_label()); |
+ decrement_loop_depth(); |
+} |
+ |
+ |
void FullCodeGenerator::EmitNewClosure(Handle<SharedFunctionInfo> info, |
bool pretenure) { |
// Use the fast case closure allocation code that allocates in new |
@@ -1081,9 +1081,8 @@ void FullCodeGenerator::VisitForInStatement(ForInStatement* stmt) { |
ForIn loop_statement(this, stmt); |
increment_loop_depth(); |
- // Get the object to enumerate over. Both SpiderMonkey and JSC |
- // ignore null and undefined in contrast to the specification; see |
- // ECMA-262 section 12.6.4. |
+ // Get the object to enumerate over. If the object is null or undefined, skip |
+ // over the loop. See ECMA-262 version 5, section 12.6.4. |
VisitForAccumulatorValue(stmt->enumerable()); |
__ LoadRoot(ip, Heap::kUndefinedValueRootIndex); |